﻿using Games.Left4Dead2;
using Nucleus.Gaming;
using Nucleus.Gaming.Controls;
using Nucleus.Gaming.Interop;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace Games
{
    public class Left4Dead2Handler : IGameHandler
    {
        protected string executablePlace;
        protected List<PlayerInfo> playas;
        protected Dictionary<string, GameOption> options;
        protected string videoFile;
        protected int titleHeight;
        protected string binFolder;
        protected string autoExec;
        protected string makeSplit;
        protected string pak01_000;
        protected string backupPak;

        public bool HideTaskBar
        {
            get { return false; }
        }
       

        public static readonly string CFGFolder = @"left4dead2\cfg";
        public static readonly string BackupFolder = @"left4dead2\backup";
        protected string levelCommand;
        protected string gameMode;

        public bool Initialize(string gameFilename, List<PlayerInfo> players, Dictionary<string, GameOption> options, List<Control> addSteps, int titleHeight)
        {
            this.executablePlace = gameFilename;
            this.playas = players;
            this.options = options;
            this.titleHeight = titleHeight - 5;

            levelCommand = ((Left4Dead2LevelSelection)addSteps[0]).LevelName;
            gameMode = ((Left4Dead2ModeSelection)addSteps[1]).ModeCommand;

            // Search for video file
            string folder = Path.GetDirectoryName(executablePlace);
            binFolder = Path.Combine(folder, "bin");
            string l4d2 = Path.Combine(folder, "left4dead2");
            pak01_000 = l4d2 + "\\pak01_000.vpk";
            backupPak = Path.Combine(folder, BackupFolder) + "\\pak01_000.vpk";
            folder = Path.Combine(folder, CFGFolder);
            autoExec = folder + "\\autoexec.cfg";
            makeSplit = folder + "\\makesplit.cfg";
            videoFile = folder + "\\video.txt";

            if (!File.Exists(videoFile))
            {
                MessageBox.Show("Could not find Video.txt file! Should be on '" + CFGFolder + "'");

                using (OpenFileDialog open = new OpenFileDialog())
                {
                    open.Filter = "Video.txt file|Video.txt";
                    if (open.ShowDialog() == DialogResult.OK)
                    {
                        videoFile = open.FileName;
                    }
                    else
                    {
                        return false;
                    }
                }
            }

            return true;
        }

        const int MYACTION_HOTKEY_ID = 1;

        public string Play()
        {
            if (!SteamUtil.IsSteamRunning())
            {
                return "Steam must be opened to Play Left 4 Dead 2 SplitScreen";
            }

            string videoCFG;
            using (Stream videoStream = new FileStream(videoFile, FileMode.Open))
            {
                videoCFG = new StreamReader(videoStream).ReadToEnd();
            }
            string originalCFG = String.Copy(videoCFG);

            int fullscreen;
            int fullscreenEnd;
            int widthPos;
            int widthPosEnd;
            int heightPos;
            int heightPosEnd;
            GetPositions(videoCFG, out widthPos, out widthPosEnd, out heightPos, out heightPosEnd, out fullscreen, out fullscreenEnd);

            // minimize everything
            User32.MinimizeEverything();

            // detect if the player is using 2 screens
            int screenIndex = -1;
            bool twoScreens = false;
            int fullWidth = 0;
            int fullHeight = 0;
            for (int i = 0; i < playas.Count; i++)
            {
                PlayerInfo player = playas[i];
                Screen scr = Screen.AllScreens[player.ScreenIndex];

                if (screenIndex == -1)
                {
                    screenIndex = player.ScreenIndex;
                    fullWidth = scr.Bounds.Width;
                    fullHeight = scr.Bounds.Height;
                }
                else
                {
                    if (screenIndex != player.ScreenIndex)
                    {
                        twoScreens = true;
                        // Add 2nd monitor
                        fullWidth += scr.Bounds.Width;
                    }
                }
            }

            string fWidth = fullWidth.ToString();
            string fHeight = fullHeight.ToString();
            string fullScr = (1).ToString();

            int dif = fWidth.Length - (widthPosEnd - widthPos);
            heightPos += dif;
            heightPosEnd += dif;

            dif += fHeight.Length - (heightPosEnd - heightPos);
            fullscreen += dif;
            fullscreenEnd += dif;

            videoCFG = videoCFG.Remove(widthPos, widthPosEnd - widthPos);
            videoCFG = videoCFG.Insert(widthPos, fWidth);
            videoCFG = videoCFG.Remove(heightPos, heightPosEnd - heightPos);
            videoCFG = videoCFG.Insert(heightPos, fHeight);
            videoCFG = videoCFG.Remove(fullscreen, fullscreenEnd - fullscreen);
            videoCFG = videoCFG.Insert(fullscreen, fullScr);

            using (Stream videoStream = new FileStream(videoFile, FileMode.Create))
            {
                StreamWriter writer = new StreamWriter(videoStream);
                writer.Write(videoCFG);
                writer.Flush();
            }

            string d3d9Path = binFolder + @"\d3d9.dll";
            if (twoScreens)
            {
                // Copy SoftTH to the game folder
                // Get the SoftTH stream
                if (!File.Exists(d3d9Path))
                {
                    using (MemoryStream stream = new MemoryStream(GamesResources.d3d9))
                    {
                        // write to bin folder
                        using (FileStream file = new FileStream(d3d9Path, FileMode.Create))
                        {
                            stream.CopyTo(file);
                        }
                    }
                }
            }
            else
            {
                // Delete SoftTH from the game folder
                FileInfo file = new FileInfo(d3d9Path);
                if (!file.IsReadOnly)
                {
                    // Only delete if it's not read only
                    if (File.Exists(d3d9Path))
                    {
                        File.Delete(d3d9Path);
                    }
                }
            }

            if (twoScreens)
            {
                FileInfo f = new FileInfo(pak01_000);
                if (!f.IsReadOnly)
                {
                    // make a backup of pak01_000, if it isn't already made
                    string dir = Path.GetDirectoryName(backupPak);
                    if (!Directory.Exists(dir))
                    {
                        Directory.CreateDirectory(dir);
                    }
                    if (!File.Exists(backupPak))
                    {
                        File.Copy(pak01_000, backupPak);
                    }

                    using (FileStream str = File.OpenRead(backupPak))
                    {
                        // Now modify some stuff
                        // Eat Point that comes at 29.664.480
                        // The 8 is at position 29.664.483

                        byte one = 49;
                        byte six = 54;

                        using (FileStream stream = new FileStream(pak01_000, FileMode.Create))
                        {
                            byte[] buff = new byte[4096];

                            int point = 29664480;

                            while (str.Position < point)
                            {
                                int length = 4096;
                                if (point - str.Position < length)
                                {
                                    length = (int)(point - str.Position);
                                }

                                str.Read(buff, 0, length);
                                stream.Write(buff, 0, length);
                            }
                            // Jumps 1 byte
                            str.Position += 1;

                            str.Read(buff, 0, 2);
                            stream.Write(buff, 0, 2);

                            // Jumps the 8
                            str.Position += 1;

                            buff[0] = one;
                            buff[1] = six;
                            stream.Write(buff, 0, 2);

                            while (stream.Position < str.Length)
                            {
                                int length = 4096;
                                if (str.Length - str.Position < length)
                                {
                                    length = (int)(str.Length - str.Position);
                                }

                                str.Read(buff, 0, length);
                                stream.Write(buff, 0, length);
                            }
                        }
                    }

                    f.IsReadOnly = true;
                }
            }
            else
            {
                if (File.Exists(backupPak))
                {
                    if (File.Exists(pak01_000))
                    {
                        FileInfo f = new FileInfo(pak01_000);
                        f.IsReadOnly = false;
                        File.Delete(pak01_000);
                    }
                    File.Copy(backupPak, pak01_000);
                }
            }

            int splitMode = 1;
            for (int i = 0; i < playas.Count; i++)
            {
                PlayerInfo player = playas[i];
                if (player.ScreenType == ScreenType.HorizontalBottom ||
                    player.ScreenType == ScreenType.HorizontalTop)
                {
                    splitMode = 1;
                }
                else
                {
                    splitMode = 2;
                }
            }

            CultureInfo c = CultureInfo.InvariantCulture;
            using (FileStream stream = new FileStream(autoExec, FileMode.Create))
            {
                StreamWriter writer = new StreamWriter(stream);
                writer.WriteLine("ss_splitmode " + splitMode.ToString(c));
                writer.WriteLine("ss_map " + levelCommand + " " + gameMode);
                writer.WriteLine("sv_allow_lobby_connect_only 0");
                string aspas = '"'.ToString();
                writer.WriteLine("bind F8 " + aspas + "exec makesplit" + aspas);

                writer.Flush();
            }

            using (FileStream stream = new FileStream(makeSplit, FileMode.Create))
            {
                StreamWriter writer = new StreamWriter(stream);
                writer.WriteLine("connect_splitscreen localhost 2");

                writer.Flush();
            }

            /* if (player.ScreenType == ScreenType.VerticalLeft ||
                    player.ScreenType == ScreenType.VerticalRight)*/

            //ss_splitmode 1 = horizontal
            //ss_splitmode 2 = vertical

            ProcessStartInfo startInfo = new ProcessStartInfo();
            startInfo.FileName = executablePlace;
            startInfo.WindowStyle = ProcessWindowStyle.Hidden;

            MessageBox.Show("Press the F8 key after the game is loaded to begin splitscreen!");

            proc = Process.Start(startInfo);

            return string.Empty;
        }

        public static void sendKeystroke(ushort k, IntPtr window)
        {
            const int WM_KEYDOWN = 0x100;
            IntPtr result3 = User32Interop.SendMessage(window, WM_KEYDOWN, ((IntPtr)k), (IntPtr)0);
        }

        bool hooked = false;

        HotkeyListener listener;
        Process proc;

        private void GetPositions(string video, out int width, out int widthEnd, out int height, out int heightEnd, out int fullscreen, out int fullscreenEnd)
        {
            width = -1;
            widthEnd = -1;
            height = -1;
            heightEnd = -1;
            fullscreen = -1;
            fullscreenEnd = -1;
            // Look for the defaultres and defaultresheight
            const string defaultRes = "defaultres";
            const string defaultResHeight = "defaultresheight";
            const string def_fullscreen = "fullscreen";

            int def = video.IndexOf(defaultRes);
            int aspas = 0;
            bool firstNumber = true;

            for (int i = def + defaultRes.Length; i < video.Length; i++)
            {
                char c = video[i];

                if (width == -1)
                {
                    if (c == '"')
                    {
                        aspas++;
                        if (aspas == 2)
                        {
                            width = i;
                        }
                    }
                }
                else
                {
                    if (StringUtil.IsNumber(c) && firstNumber)
                    {
                        firstNumber = false;
                        width = i;
                    }
                    if (!StringUtil.IsNumber(c))
                    {
                        widthEnd = i;
                        break;
                    }
                }
            }

            def = video.IndexOf(defaultResHeight);
            aspas = 0;
            firstNumber = true;
            for (int i = def; i < video.Length; i++)
            {
                char c = video[i];
                if (height == -1)
                {
                    if (c == '"')
                    {
                        aspas++;
                        if (aspas == 2)
                        {
                            height = i;
                        }
                    }
                }
                else
                {
                    if (StringUtil.IsNumber(c) && firstNumber)
                    {
                        firstNumber = false;
                        height = i;
                    }
                    if (!StringUtil.IsNumber(c))
                    {
                        heightEnd = i;
                        break;
                    }
                }
            }
            def = video.IndexOf(def_fullscreen);
            aspas = 0;
            firstNumber = true;
            for (int i = def; i < video.Length; i++)
            {
                char c = video[i];
                if (fullscreen == -1)
                {
                    if (c == '"')
                    {
                        aspas++;
                        if (aspas == 2)
                        {
                            fullscreen = i;
                        }
                    }
                }
                else
                {
                    if (StringUtil.IsNumber(c) && firstNumber)
                    {
                        firstNumber = false;
                        fullscreen = i;
                    }
                    if (!StringUtil.IsNumber(c))
                    {
                        fullscreenEnd = i;
                        break;
                    }
                }
            }
        }

        public void Update(int delayMS)
        {
        }

        public void End()
        {
            if (listener != null)
            {
                if (hooked)
                {
                    User32Interop.UnregisterHotKey(listener.Handle, MYACTION_HOTKEY_ID);
                }
                listener.Dispose();
                listener = null;
            }

            if (!string.IsNullOrEmpty(autoExec))
            {
                if (File.Exists(autoExec))
                {
                    using (FileStream stream = new FileStream(autoExec, FileMode.Create))
                    {
                        StreamWriter writer = new StreamWriter(stream);

                        // Empties auto exec so the game won't try loading the multiplayer level
                        writer.Write("");
                        writer.Flush();
                    }
                }
            }
        }
    }
}
