﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Nucleus.Gaming
{
    public class UserGameProfile
    {
        public int Players;
        public Dictionary<int, ScreenType> Positions;
        public Dictionary<string, GameOption> Options;

        public static UserGameProfile Read(BinaryReader reader)
        {
            UserGameProfile prof = new UserGameProfile();
            int players = reader.ReadInt32();

            prof.Positions = new Dictionary<int, ScreenType>();
            for (int i = 0; i < players; i++)
            {
                int playa = reader.ReadInt32();
                ScreenType type = (ScreenType)reader.ReadInt32();
                prof.Positions.Add(playa, type);
            }

            bool options = reader.ReadBoolean();
            if (options)
            {
                int count = reader.ReadInt32();

                prof.Options = new Dictionary<string, GameOption>();
                for (int i = 0; i < count; i++)
                {
                    string key = reader.ReadString();
                    string value = reader.ReadString();
                    GameOption opt = new GameOption("", "", value);
                    prof.Options.Add(key, opt);
                }
            }


            return prof;
        }

        public static void Write(BinaryWriter writer, UserGameProfile prof)
        {
            writer.Write(prof.Players);

            var positions = prof.Positions;
            writer.Write(positions.Count);
            foreach (var p in positions)
            {
                writer.Write(p.Key);
                writer.Write((int)p.Value);
            }

            var options = prof.Options;
            writer.Write(options != null);
            if (options != null)
            {
                writer.Write(options.Count);

                foreach (var p in options)
                {
                    writer.Write(p.Key);
                    object ob = p.Value.Value;
                    // I Sure as hell know this ain't no way to serialize this type of object,
                    // but currently I cannot insert my Game Engine to this project, as we did not launch it yet.
                    // When we do launch, this will work perfeclty as I will be using it's system for serializating
                    writer.Write(ob.ToString());
                }
            }
        }
    }
}
