﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Nucleus.Gaming
{
    public static class ViewportUtil
    {
        public static void GetPlayerViewport(PlayerInfo player, int titleHeight, out int width, out int height, out Point location)
        {
            width = -1;
            height = -1;
            location = new Point();

            Screen[] all = Screen.AllScreens;
            Screen first = all[0];
            Rectangle fbounds = first.Bounds;

            Screen screen = all[player.ScreenIndex];
            Rectangle bounds = screen.Bounds;

            switch (player.ScreenType)
            {
                case ScreenType.Fullscreen:
                    {
                        width = bounds.Width;
                        height = bounds.Height;
                        location = new Point(screen.Bounds.X, screen.Bounds.Y);
                    }
                    break;
                case ScreenType.HorizontalTop:
                    {
                        width = screen.Bounds.Width;
                        height = (screen.Bounds.Height / 2) - titleHeight;
                        location = new Point(screen.Bounds.X, screen.Bounds.Y);
                    }
                    break;
                case ScreenType.HorizontalBottom:
                    {
                        width = screen.Bounds.Width;
                        height = (screen.Bounds.Height / 2) - titleHeight;
                        location = new Point(screen.Bounds.X, screen.Bounds.Y + (screen.Bounds.Height / 2));
                    }
                    break;
                case ScreenType.VerticalLeft:
                    {
                        width = screen.Bounds.Width / 2;
                        height = screen.Bounds.Height;
                        location = new Point(screen.Bounds.X, screen.Bounds.Y);
                    }
                    break;
                case ScreenType.VerticalRight:
                    {
                        width = screen.Bounds.Width / 2;
                        height = screen.Bounds.Height;
                        location = new Point(screen.Bounds.X + (screen.Bounds.Width / 2), screen.Bounds.Y);
                    }
                    break;
                case ScreenType.TopLeft:
                    {
                        width = screen.Bounds.Width / 2;
                        height = screen.Bounds.Height / 2;
                        location = new Point(screen.Bounds.X, screen.Bounds.Y);
                    }
                    break;
                case ScreenType.TopRight:
                    {
                        width = screen.Bounds.Width / 2;
                        height = screen.Bounds.Height / 2;
                        location = new Point(screen.Bounds.X + width, screen.Bounds.Y);
                    }
                    break;
                case ScreenType.BottomLeft:
                    {
                        width = screen.Bounds.Width / 2;
                        height = screen.Bounds.Height / 2;
                        location = new Point(screen.Bounds.X, screen.Bounds.Y + height);
                    }
                    break;
                case ScreenType.BottomRight:
                    {
                        width = screen.Bounds.Width / 2;
                        height = screen.Bounds.Height / 2;
                        location = new Point(screen.Bounds.X + width, screen.Bounds.Y + height);
                    }
                    break;
            }

        }
    }
}
