﻿using Nucleus.Gaming;
using Nucleus.Gaming.Controls;
using SplitTool.Controls;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SplitTool
{
    public partial class GamesViewer : Form
    {
        protected GameManager gameManager;
        protected GameInfo gameInfo;
        protected UserGameInfo info;

        protected Dictionary<int, Control> steps;
        protected int step;
        protected List<Control> addSteps;

        protected IGameHandler handler;
        protected static readonly int DefaultSteps = 2;

        bool initialized = false;
        public GamesViewer()
        {
            AppDomain.CurrentDomain.UnhandledException += CurrentDomain_UnhandledException;

            this.ControlAdded += GamesViewer_ControlAdded;
            this.MouseHover += GamesViewer_MouseHover;

            addSteps = new List<Control>();

            InitializeComponent();

            this.ActiveControl = this.label_Library;

            gameManager = new GameManager();
            list_Games.DataSource = gameManager.User.Games;

            steps = new Dictionary<int, Control>();
            steps.Add(0, this.monitorControl1);
            steps.Add(1, this.playerOptions1);

            initialized = true;
            list_Games.SelectedIndex = -1;
            if (list_Games.Items.Count > 0)
            {
                list_Games.SelectedIndex = 0;
            }

            num_Players_ValueChanged(num_Players, EventArgs.Empty);
        }

        void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            MessageBox.Show(e.ExceptionObject.ToString());

            try
            {
                gameManager.End();
                if (handler != null)
                {
                    handler.End();
                }
            }
            catch { }

#if DEBUG
            try
            {
                if (e.ExceptionObject is Exception)
                {
                    SaveException((Exception)e.ExceptionObject);
                }
            }
            catch { }
#endif
        }

        private void SaveException(Exception e)
        {
            // Save Log of the Call Stack
            string log = GetLog(e);
            Assembly myAss = Assembly.GetEntryAssembly();
            string dir = Path.GetDirectoryName(myAss.Location);
            DateTime time = DateTime.Now;
            string logLoc = Path.Combine(dir, "Log " + time.Hour + "-" + time.Minute + "-" + time.Second) + ".log";

            using (FileStream stream = new FileStream(logLoc, FileMode.Create))
            {
                StreamWriter writer = new StreamWriter(stream);
                writer.Write(log);
                writer.Flush();
            }
        }

        public string GetLog(Exception ex)
        {
            string log = "";
            Assembly myAss = Assembly.GetEntryAssembly();

            DateTime now = DateTime.Now;

            log += "// Nucleus Exception log";
            log += Environment.NewLine;
            log += now.ToShortDateString();
            log += Environment.NewLine;
            log += now.ToShortTimeString();
            log += Environment.NewLine;

            log += myAss.FullName;
            log += Environment.NewLine;

            log += ex.ToString();
            log += Environment.NewLine;
            log += ex.GetType().FullName;
            log += Environment.NewLine;
#if WINRT
            log += ex.GetType().GetTypeInfo().Assembly.FullName;
#else
            log += ex.GetType().Assembly.FullName;
#endif

            return log;
        }

        void GamesViewer_ControlAdded(object sender, ControlEventArgs e)
        {
            e.Control.MouseHover += Control_MouseHover;
        }

        void GamesViewer_MouseHover(object sender, EventArgs e)
        {
            ClickedControl(this);
        }
        void Control_MouseHover(object sender, EventArgs e)
        {
            ClickedControl((Control)sender);
        }

        private void txt_Search_MouseEnter(object sender, EventArgs e)
        {
            label_Search.ForeColor = Color.FromArgb(200, 200, 200);
        }
        private void txt_Search_MouseLeave(object sender, EventArgs e)
        {
            label_Search.ForeColor = Color.FromArgb(150, 150, 150);
        }

        private void txt_Search_MouseClick(object sender, MouseEventArgs e)
        {
            label_Search.Visible = false;
        }


        protected void ClickedControl(Control c)
        {
            if (c != txt_Search && !txt_Search.Focused)
            {
                label_Search.Visible = true;
            }
        }

        private void btn_AddGame_Click(object sender, EventArgs e)
        {
            FindGameForm form = new FindGameForm(gameManager);
            if (form.ShowDialog() == DialogResult.OK)
            {
                // Update the game list?
                list_Games.DataSource = null;
                list_Games.DataSource = gameManager.User.Games;
            }
        }

        private void list_Games_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!initialized)
            {
                return;
            }

            if (list_Games.SelectedItem is UserGameInfo)
            {
                step = 0;
                info = (UserGameInfo)list_Games.SelectedItem;
                gameInfo = gameManager.Games[info.GameGuid];

                if (!File.Exists(info.ExecutablePath))
                {
                    MessageBox.Show("Game Executable not Found. Deleting game from your library...");
                    
                    gameManager.User.Games.Remove(info);
                    gameManager.UpdateUserProfile();

                    info = null;
                    gameInfo = null;

                    list_Games.DataSource = null;
                    list_Games.SelectedIndex = -1;
                    list_Games.DataSource = gameManager.User.Games;

                    if (list_Games.Items.Count > 0)
                    {
                        list_Games.SelectedIndex = 0;
                    }

                    return;

                }

                this.label_GameName.Text = info.GameName;
                this.label_MaxPlayers.Text = gameInfo.MaxPlayers.ToString(CultureInfo.InvariantCulture);

                this.num_Players.Maximum = gameInfo.MaxPlayers;
                this.num_Players.Value = gameInfo.MaxPlayers;
                monitorControl1.UpdatePlayerCount((int)num_Players.Value, gameInfo);
                playerOptions1.UpdateItems(gameInfo);

                addSteps.Clear();
                if (gameInfo.Steps != null)
                {
                    // add the steps
                    int count = steps.Count;
                    for (int i = 0; i < gameInfo.Steps.Length; i++)
                    {
                        Control s = (Control)Activator.CreateInstance(gameInfo.Steps[i]);
                        s.Location = this.monitorControl1.Location;
                        s.Size = this.monitorControl1.Size;
                        s.Anchor = this.monitorControl1.Anchor;
                        borderPanel1.Controls.Add(s);
                        steps.Add(count + i, s);

                        addSteps.Add(s);
                    }
                }
                else
                {
                    // delete any olds
                    if (steps.Count > DefaultSteps)
                    {
                        int count = steps.Count;
                        for (int i = DefaultSteps; i < count; i++)
                        {
                            borderPanel1.Controls.Remove(steps[i]);
                            steps.Remove(i);
                        }
                    }
                }

                // extract icon
                if (this.game_Box.Image != null)
                {
                    this.game_Box.Image.Dispose();
                }

                using (System.Drawing.Icon sysicon = Icon.ExtractAssociatedIcon(info.ExecutablePath))
                {
                    using (MemoryStream str = new MemoryStream())
                    {
                        sysicon.Save(str);
                        str.Position = 0;
                        game_Box.Image = Image.FromStream(str);
                    }
                }
            }
        }

        private void num_Players_ValueChanged(object sender, EventArgs e)
        {
            monitorControl1.UpdatePlayerCount((int)num_Players.Value, this.gameInfo);
            step = 0;
            UpdateStep();
        }

        private void btn_Steam_Click(object sender, EventArgs e)
        {

        }

        private void button_Next_Click(object sender, EventArgs e)
        {
            step++;
            step = Math.Min(step, this.steps.Count - 1);
            UpdateStep();
        }

        private void btn_Previous_Click(object sender, EventArgs e)
        {
            step--;
            step = Math.Max(0, step);
            UpdateStep();
        }

        private void UpdateStep()
        {
            btn_Next.Visible = step < steps.Count - 1;
            btn_Previous.Visible = step > 0;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            var st = steps[step];

            int proceed = 0;
            foreach (var ste in steps)
            {
                if (ste.Value is ICanProceed)
                {
                    ICanProceed proo = (ICanProceed)ste.Value;
                    if (proo.CanProceed)
                    {
                        proceed++;
                    }
                }
                else
                {
                    throw new Exception("Must be ICanProceed");
                }

                ste.Value.Visible = ste.Key == step;
            }
            button_Play.Enabled = proceed == steps.Count || this.handler != null;

            if (st is ICanProceed)
            {
                ICanProceed proc = (ICanProceed)st;
                if (step != steps.Count - 1)
                {
                    btn_Next.Enabled = proc.CanProceed;
                }
            }
            else
            {
                throw new Exception("Must be ICanProceed");
            }

            if (handler != null)
            {
                handler.Update(timer1.Interval);
            }
        }

        private void button_Play_Click(object sender, EventArgs e)
        {
            if (handler == null)
            {
                if (this.info == null)
                {
                    return;
                }
                string starter = this.info.ExecutablePath;

                Type t = gameInfo.HandlerType;
                handler = (IGameHandler)Activator.CreateInstance(t);

                Rectangle screenRectangle = RectangleToScreen(this.ClientRectangle);
                int titleHeight = screenRectangle.Top - this.Top;
                bool worked = handler.Initialize(starter, this.monitorControl1.Players, gameInfo.Options, addSteps, titleHeight);

                if (worked)
                {
                    string error = handler.Play();

                    if (string.IsNullOrEmpty(error))
                    {
                        button_Play.Text = "END";
                        gameManager.Play(handler);
                    }
                    else
                    {
                        handler = null;
                        MessageBox.Show(error);
                    }
                }
                else
                {
                    handler = null;
                }
            }
            else
            {
                handler.End();
                handler = null;
                gameManager.End();

                button_Play.Text = "PLAY";
            }
        }

        private void GamesViewer_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (handler != null)
            {
                handler.End();
                this.handler = null;
            }
            gameManager.End();
        }

        private void GamesViewer_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (handler != null)
            {
                handler.End();
                this.handler = null;
            }
            gameManager.End();
        }

       

     
    }
}
